/*
* =====================================================================
*  Filename: TradingSystemDlg.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "stdafx.h"
#include <map>
#include "NumEdit.h"
#include "Trader.h"

class CAutoTraderDlg;

/**
\typedef ATL::IDispEventImpl<1, CAutoTraderDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CAutoTraderDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

// CAutoTraderDlg dialog
class CAutoTraderDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   /// \brief CAutoTraderDlg constructor.
   /// \param pParent - parent window
   explicit CAutoTraderDlg(
                  CWnd* pParent = NULL); 

   /// \enum Dialog Data
   enum { IDD = IDD_AUTO_TRADER_DIALOG };

   /// \brief Dialog icon.
   HICON m_hIcon;

private:

   /// \brief DDX/DDV support.
   virtual void DoDataExchange(
                  CDataExchange* pDX);

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the framework shall set the input 
   /// \return focus to the first control of the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog from closing on Enter
   /// \brief and catch Enters in tag name and tag value edit controls.
   virtual void OnOK();

private:

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief The system calls this function to obtain the cursor to
   /// \brief display while the user drags the minimized window.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Used to unadvise from CQGCEL events and shutdown it.
   afx_msg void OnDestroy();

   /// \brief Adds new auto trader.
   afx_msg void OnBnClickedBtnAdd();

   /// \brief Removes selected auto trader.
   afx_msg void OnBnClickedBtnRemove();

   /// \brief Used to Activate/inactivate auto trader.
   /// \param pNMHDR - contains information about a notification message
   /// \param pResult - signed result of message processing
   afx_msg void OnLvnItemchangedLsvAutoTraders(
                  NMHDR *pNMHDR,
                  LRESULT *pResult);

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CAutoTraderDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 40, OnTradingSystemDefinitionsResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 49, OnTradingSystemResolved)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 50, OnTradingSystemAddNotification)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 51, OnTradingSystemUpdateNotification)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 17, OnOrderChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
   END_SINK_MAP()

private:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - string describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred in the connection with the GW server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred in the connection with the CQG data server.
   /// \param newStatus - new connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when account/position is changed (in 'broad' sense of change).
   /// \param change_type - denotes change type
   /// \param cqg_account - the account affected, or Nothing/NOT_VALID for actAccountsReloaded
   /// \param cqg_position - the position affected, or Nothing/NOT_VALID for actAccountsReloaded
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                  eAccountChangeType change_type,
                  ICQGAccount* cqg_account,
                  ICQGPosition* cqg_position);

   /// \brief Fired when a new instrument is resolved and subscribed or the requested symbol is not available.
   /// \param symbol - Requested symbol
   /// \param cqg_instrument - Subscribed instrument object
   /// \param cqg_error - Error occured during a symbol resolution process
   /// \return S_OK on success
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error);


   /// \brief Fired when the definitions of tradingSystems are resolved
   /// \brief or when some error occurred during request processing.
   /// \param cqg_trading_system_definitions - collection of trading system definitions
   /// \param cqg_error - CQGError object describing last error occurred during the request processing
   /// \return S_OK
   STDMETHOD(OnTradingSystemDefinitionsResolved)(
                  ICQGTradingSystemDefinitions* cqg_trading_system_definitions, 
                  ICQGError* cqg_error);

   /// \brief Fired when the trading system (CQGTradingSystem) is resolved
   /// \brief or when some error occurred during trading system request processing.
   /// \param cqg_trading_system - reference to resolved CQGTradingSystem
   /// \param cqg_error - CQGError object describing last error occurred during processing the request.
   /// \return S_OK
   STDMETHOD(OnTradingSystemResolved)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGError* cqg_error);

   /// \brief Fired when trade rows and/or statistics are added to trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_add_info - object containing information about the additions
   /// \return S_OK
   STDMETHOD(OnTradingSystemAddNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemAddInfo* cqg_trading_system_add_info);

   /// \brief Fired when trade rows and/or statistics are updated in a trading system.
   /// \param cqg_trading_system - reference to changed CQGTradingSystem
   /// \param cqg_trading_system_update_info - object containing information about the updates
   /// \return S_OK
   STDMETHOD(OnTradingSystemUpdateNotification)(
                  ICQGTradingSystem* cqg_trading_system,
                  ICQGTradingSystemUpdateInfo* cqg_trading_system_update_info);

   /// \brief Fired when new order was added, order status was 
   /// \brief changed or the order was removed.
   /// \param change - The type of the occurred change
   /// \param order - CQGOrder object representing the order to which the change refers
   /// \param oldProperties - CQGOrderProperties collection representing 
   /// \param the old values of the changed order properties
   /// \param fill - CQGFill object representing the last fill of the order
   /// \param cqgerr - CQGError object to describe the last error, if any
   /// \return S_OK
   STDMETHOD(OnOrderChanged)(
                  eChangeType change, 
                  ICQGOrder* order, 
                  ICQGOrderProperties* oldProperties, 
                  ICQGFill* fill, 
                  ICQGError* cqgerr);

private:

   /// \enum eAutoTradersColumns. Represents auto traders columns in the list.
   enum eAutoTradersColumns;

   ///// \enum eExecutedOrdersColumns. Represents executed orders columns in the list.
   enum eExecutedOrdersColumns;

private:

   /// \brief Initializes CQGCEL object, and starts the CQGCEL.
   void InitializeCQGCEL();

   /// \brief Initializes all controls.
   void InitializeControls();

   /// \brief Initializes auto traders list control's columns and styles.
   void InitializeAutoTradersList();

   ///// \brief Initializes executed orders list control's columns and styles.
   void InitializeExecutedOrdersList();

   /// \brief Finds specified trading system's row in the list.
   /// \param tradingSystemID - trading system ID
   /// \return Row of the item if successful or -1 otherwise.
   int FindAutoTraderRow(
                  const _bstr_t& tradingSystemID);

   /// \brief Finds specified order's row in the list.
   /// \param orderID - order ID
   /// \return Row of the item if successful or -1 otherwise.
   int FindOrderRow(
                  const _bstr_t& orderID);

   /// \brief Finds specified trading system's Trader object.
   /// \param tradingSystemID - trading system ID
   /// \return Trader object.
   Trader& GetAutoTraderByID(
                  const _bstr_t& tradingSystemID);

   /// \brief Shows specified order in the executed orders' list.
   /// \param autoTraderName - auto trader name which will be shown
   /// \param order - order which properties will be shown
   void ShowOrder(
                  const CString& autoTraderName,
                  const ICQGOrderPtr& order);

   /// \brief Updates specified order in the executed orders' list.
   /// \param row - order's row in the list view
   void UpdateOrder(
                  int row,
                  const ICQGOrderPtr& order);

   /// \brief Clears all data(parameters\outputs\records) from the form.
   void ClearAllData();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - variant value
   /// \param valueType - variant type
   /// \return String representation of the value.
   CString GetValueAsString(
                  ATL::CComVariant value,
                  VARTYPE valueType = VT_EMPTY) const;

   /// \brief Enables/disables controls on the form based on connection status and other parameters.
   void EnableControls();

   /// \brief Called when failed to subscribe to instrument based on symbol name.
   /// \param symbol_ - symbol which was passed to NewInstrument
   void OnResolutionFailed(BSTR symbol);
   
private:

   /// \brief CQGCEL object.
   ICQGCELPtr m_spCQGCEL;

   /// \brief Selected trading system object.
   ICQGTradingSystemPtr m_spTradingSystem;

   /// \brief Definitions resolved status.
   bool m_DefinitionsResolved;

   /// \brief Accounts resolved status.
   bool m_AccountsResolved;

   /// \brief Map of auto traders.
   typedef std::map<_bstr_t, Trader> Traders;
   Traders m_TradersMap;

   /// \brief Selected auto trader.
   Trader* m_SelectedTrader;

private:

   /// \brief Executed orders count that will be shown in the list.
   static const long AUTO_TRADERS_COUNT;

private:

   /// \brief Data connection indicator Label control.
   CStatic m_lblDataConnection;

   /// \brief GW connection indicator Label control.
   CStatic m_lblGwConnection;

   /// \brief Trading systems definitions status Label control.
   CStatic m_lblDefinitionsStatus;

   /// \brief Accounts status Label control.
   CStatic m_lblAccountsStatus;

   /// \brief Trading system definitions ComboBox control.
   CComboBox m_cmbDefinitions;

   /// \brief Auto traders ListView control.
   CListCtrl m_lvAutoTraders;
   
   ///// \brief Executed orders ListView control.
   CListCtrl m_lvExecutedOrders;

   /// \brief Add auto trader Button control.
   CButton m_btnAdd;

   /// \brief Remove auto trader Button control.
   CButton m_btnRemove;

};

enum CAutoTraderDlg::eAutoTradersColumns
{
   atcOn,
   atcTurnedOnAt,
   atcOpenPositions,
   atcName,
   atcTradingSystem,
   atcTradingSystemStatus,
   atcSymbol,
   atcSymbolStatus,
   atcAccountID,
   atcAccountName,
   atcFullName,
   atcPreferences,
   atcParameters,
   atcTradingSystemID
};

enum CAutoTraderDlg::eExecutedOrdersColumns
{
   eocAutoTradeName,
   eocAccount,
   eocInstrument,
   eocSide,
   eocQuantity,
   eocFilledQuantity,
   eocGWStatus,
   eocState,
   eocPlaceTime,
   eocOrderType,
   eocOrderID,
   eocUEName,
   eocError,
   eocGUID
};
