// TradingSystemReqDlg.cpp : implementation file
//

#include "stdafx.h"
#include <vector>
#include "Common.h"
#include "Helper.h"
#include "AutoTrader.h"
#include "AutoTraderReqDlg.h"

CAutoTraderReqDlg::CAutoTraderReqDlg(
                              const ICQGTradingSystemRequestPtr& spRequest,
                              const ICQGAccountsPtr& spAccounts,
                              CWnd* pParent /*=NULL*/)
                              : CDialog(CAutoTraderReqDlg::IDD, pParent),
                                m_spRequest(spRequest),
                                m_spAccounts(spAccounts),
                                m_BaseExpression(_T("")),
                                m_EqualizeCloses(FALSE),
                                m_DailyFromIntraday(FALSE),
                                m_CustomSession(FALSE)
{
}

void CAutoTraderReqDlg::DoDataExchange(
                              CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);

   DDX_Text(pDX, IDC_ED_BASE_EXPRESSION, m_BaseExpression);
   DDX_Control(pDX, IDC_ED_TRADER_NAME, m_edTraderName);

   DDX_Control(pDX, IDC_CMB_ACCOUNTS, m_cmbAccounts);

   DDX_Control(pDX, IDC_LSV_PARAMETERS, m_lvParameters);
   DDX_Control(pDX, IDC_ED_PARAMETER_VALUE, m_edParameterValue);
   DDX_Control(pDX, IDC_BTN_CHANGE_VALUE, m_btnChangeValue);

   DDX_Control(pDX, IDC_CMB_HISTORICAL, m_cmbHistorical);

   DDX_Control(pDX, IDC_ED_INTRADAY, m_edIntraday);
   DDX_Control(pDX, IDC_SPN_INTRADAY, m_spnIntraday);

   DDX_Control(pDX, IDC_CMB_CONTINUATION, m_cmbContinuation);

   DDX_Control(pDX, IDC_CHK_EQUALIZE_CLOSES, m_chkEqualizeCloses);
   DDX_Check(pDX, IDC_CHK_EQUALIZE_CLOSES, m_EqualizeCloses);

   DDX_Control(pDX, IDC_ED_DAYS_BEFORE_EXP, m_edDaysBeforeExp);
   DDX_Control(pDX, IDC_SPN_DAYS_BEFORE_EXP, m_spnDaysBeforeExp);

   DDX_Check(pDX, IDC_CHK_DAILY_FROM_INTR, m_DailyFromIntraday);

   DDX_Control(pDX, IDC_ED_SESSIONS_FILTER, m_edSessionsFilter);
   DDX_Control(pDX, IDC_SPN_SESSIONS_FILTER, m_spnSessionsFilter);
   
   DDX_Control(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_edCustomSessionsFilter);
   DDX_Text(pDX, IDC_ED_CUSTOM_SESSIONS_FILTER, m_CustomSessionsFilter);

   DDX_Check(pDX, IDC_CHK_CUSTOM_SESSION, m_CustomSession);
}

BOOL CAutoTraderReqDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      InitializeControls();
   }
   catch (_com_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return TRUE;
}

BEGIN_MESSAGE_MAP(CAutoTraderReqDlg, CDialog)
   ON_BN_CLICKED(IDOK, OnBnClickedOk)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_SESSIONS_FILTER, OnDeltaposSpnSessionsFilter)
   ON_BN_CLICKED(IDC_CHK_CUSTOM_SESSION, OnBnClickedChkCustomSession)
   ON_CBN_SELCHANGE(IDC_CMB_CONTINUATION, OnCbnSelChangeCmbContinuation)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_INTRADAY, OnDeltaposSpnIntraday)
   ON_CBN_SELCHANGE(IDC_CMB_HISTORICAL, OnCbnSelChangeCmbHistorical)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_DAYS_BEFORE_EXP, OnDeltaposSpnDaysBeforeExp)
   ON_BN_CLICKED(IDC_BTN_CHANGE_VALUE, OnBnClickedBtnChangeValue)
   ON_NOTIFY(LVN_ITEMCHANGED, IDC_LSV_PARAMETERS, OnLvnItemchangedLsvParameters)
END_MESSAGE_MAP()

void CAutoTraderReqDlg::InitializeControls()
{
   m_edParameterValue.SetNegativeAllowed(true);
   m_edParameterValue.SetValue(0);
   m_edParameterValue.SetMaxFractionalPartLength(CHelper::PARAM_FRAC_PART_LENGTH);

   InitializeParametersList();

   InitializeAccountsCombo();
   InitializeHistoricalCombo();
   InitializeContinuationCombo();

   // Set default parameters
   SetDefaultParameters();

   AddParameters();
}

void CAutoTraderReqDlg::InitializeParametersList()
{
   m_lvParameters.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvParameters.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvParameters.InsertColumn(pcName, _T("Name"), LVCFMT_LEFT, 80);
   m_lvParameters.InsertColumn(pcComment, _T("Comment"), LVCFMT_LEFT, 160);
   m_lvParameters.InsertColumn(pcType, _T("Type"), LVCFMT_LEFT, 50);
   m_lvParameters.InsertColumn(pcDefaultValue, _T("Def. Value"), LVCFMT_LEFT, 70);
   m_lvParameters.InsertColumn(pcValue, _T("Curr. Value"), LVCFMT_LEFT, 70);
}

CString CAutoTraderReqDlg::GetAutoTraderName() const
{
   return m_TraderName;
}

long CAutoTraderReqDlg::GetAccountID() const
{
   return m_AccountID;
}

void CAutoTraderReqDlg::SetDefaultParameters()
{
   UpdateData();

   // Get default values form request
   long defIntradayPeriod = m_spRequest->GetIntradayPeriod();

   eHistoricalPeriod defHistoricalPeriod = m_spRequest->GetHistoricalPeriod();
   ATL::CComVariant defSessionsFilter = m_spRequest->GetSessionsFilter();
   eTimeSeriesContinuationType defContinuation = m_spRequest->GetContinuation();
   VARIANT_BOOL defEqualizeCloses = m_spRequest->GetEqualizeCloses();
   long defDaysBeforeExp = m_spRequest->GetDaysBeforeExpiration();
   eSessionFlag defSessionFlags = m_spRequest->GetSessionFlags();

   // Set controls default values
   m_edIntraday.SetValue(defIntradayPeriod);
   CHelper::SetComboBoxItem(m_cmbHistorical, (DWORD_PTR)defHistoricalPeriod);
   CHelper::SetComboBoxItem(m_cmbContinuation, (DWORD_PTR)defContinuation);
   m_EqualizeCloses = defEqualizeCloses;
   m_edDaysBeforeExp.SetValue(defDaysBeforeExp);
   SetSessionFlags(defSessionFlags);
   if (VT_UI4 == defSessionsFilter.vt)
   {
      m_CustomSession = FALSE;
      m_edSessionsFilter.SetValue(defSessionsFilter.uintVal);
      m_CustomSessionsFilter = _T("");
   }
   else if (VT_BSTR == defSessionsFilter.vt)
   {
      m_CustomSession = TRUE;
      m_edSessionsFilter.SetValue(0);
      m_CustomSessionsFilter = defSessionsFilter.bstrVal;
   }

   OnCbnSelChangeCmbHistorical();
   OnCbnSelChangeCmbContinuation();
   OnBnClickedChkCustomSession();

   UpdateData(FALSE);
}

void CAutoTraderReqDlg::AddParameters()
{
   // Get definition from request
   ICQGTradingSystemDefinitionPtr spTradingSystemDefinition = m_spRequest->GetDefinition();

   // Add all parameters into the list
   m_lvParameters.DeleteAllItems();

   ICQGParameterDefinitionsPtr spParameterDefinitions = spTradingSystemDefinition->GetParameterDefinitions();

   long count = spParameterDefinitions->GetCount();

   for (long i = 0; i < count; ++i)
   {
      ICQGParameterDefinitionPtr spParameterDefinition = spParameterDefinitions->GetItem(i);
      _bstr_t name = spParameterDefinition->GetName();
      _bstr_t comment = spParameterDefinition->GetComment();
      eUserFormulaParameterType parameterType = spParameterDefinition->GetType();
      ATL::CComVariant defaultValue = spParameterDefinition->GetDefaultValue();

      const int row = m_lvParameters.InsertItem(m_lvParameters.GetItemCount(), _T(""));
      m_lvParameters.SetItemText(row, pcName, ATL::CW2T(name));
      m_lvParameters.SetItemText(row, pcComment, ATL::CW2T(comment));
      m_lvParameters.SetItemText(row, pcType, CHelper::ParameterTypeToString(parameterType));
      m_lvParameters.SetItemText(row, pcDefaultValue,
                        Common::VariantToString(defaultValue, CHelper::PARAM_FRAC_PART_LENGTH));
      m_lvParameters.SetItemText(row, pcValue,
                        Common::VariantToString(defaultValue, CHelper::PARAM_FRAC_PART_LENGTH));
   }
}

void CAutoTraderReqDlg::OnBnClickedOk()
{
   try
   {
      UpdateData();

      m_edTraderName.GetWindowText(m_TraderName);
      if (m_TraderName.IsEmpty())
      {
         AfxMessageBox(_T("Auto trader name is not specified."), MB_ICONSTOP);
         return;
      }

      if (m_cmbAccounts.GetCurSel() == CB_ERR )
      {
         AfxMessageBox(_T("Account is not specified."), MB_ICONSTOP);
         return;
      }

      m_AccountID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

      // Set request parameters
      _bstr_t baseExpression(m_BaseExpression);
      m_spRequest->PutBaseExpression(baseExpression);

      m_spRequest->PutRangeStart(ATL::CComVariant(0));
      m_spRequest->PutRangeEnd(ATL::CComVariant(-1));

      // Get historical period
      eHistoricalPeriod historicalPeriod = 
            static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      if (hpUndefined == historicalPeriod)
      {
         m_spRequest->PutIntradayPeriod(m_edIntraday.GetValue());
      }
      else
      {
         m_spRequest->PutHistoricalPeriod(historicalPeriod);
      }

      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      m_spRequest->PutContinuation(continuationType);
      m_spRequest->PutEqualizeCloses(m_EqualizeCloses ? VARIANT_TRUE : VARIANT_FALSE);
      m_spRequest->PutDaysBeforeExpiration(m_edDaysBeforeExp.GetValue());

      if (m_CustomSession)
      {
         m_spRequest->PutSessionsFilter(ATL::CComVariant(m_CustomSessionsFilter));
      }
      else
      {
         m_spRequest->PutSessionsFilter(ATL::CComVariant(m_edSessionsFilter.GetValue()));
      }

      m_spRequest->PutSessionFlags(GetSessionFlags());
      m_spRequest->ExcludeAllStatistics();
      m_spRequest->IncludeStatistic(tssOpenPosition);

      // Close request window
      OnOK();
   }
   catch (_com_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderReqDlg::InitializeAccountsCombo()
{
   long count = m_spAccounts->GetCount();

   if (count == 0)
   {
      return;
   }

   // Iterate over accounts collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   HRESULT hr = m_spAccounts->get__NewEnum(&spEnumVariant);
   ATLASSERT(SUCCEEDED(hr));

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   ATLASSERT(SUCCEEDED(hr));

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);
      ATL::CComPtr<ICQGAccount> spAccount;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
      ATLASSERT(SUCCEEDED(hr));

      // Create string for specified account to add to the combo
      CString accString;
      CString accountName = ATL::CW2T(spAccount->GetGWAccountName());
      accString.Format(_T("%s (%s)"), static_cast<LPCTSTR>(accountName), 
         static_cast<LPCTSTR>(ATL::CW2T(spAccount->GetFcmAccountID())));

      // Get account ID to map with item
      const long accID = spAccount->GetGWAccountID();

      // Add new account name to the combo and map ID
      const int row = m_cmbAccounts.AddString(accString);
      m_cmbAccounts.SetItemData(row, accID);
   }

   // Select the first account in the list
   m_cmbAccounts.SetCurSel(0);
}

void CAutoTraderReqDlg::InitializeHistoricalCombo()
{
   int row = m_cmbHistorical.AddString(_T("Undefined"));
   m_cmbHistorical.SetItemData(row, hpUndefined);

   row = m_cmbHistorical.AddString(_T("Daily"));
   m_cmbHistorical.SetItemData(row, hpDaily);

   row = m_cmbHistorical.AddString(_T("Weekly"));
   m_cmbHistorical.SetItemData(row, hpWeekly);

   row = m_cmbHistorical.AddString(_T("Monthly"));
   m_cmbHistorical.SetItemData(row, hpMonthly);

   row = m_cmbHistorical.AddString(_T("Quarterly"));
   m_cmbHistorical.SetItemData(row, hpQuarterly);

   row = m_cmbHistorical.AddString(_T("Semiannual"));
   m_cmbHistorical.SetItemData(row, hpSemiannual);

   row = m_cmbHistorical.AddString(_T("Yearly"));
   m_cmbHistorical.SetItemData(row, hpYearly);
}

void CAutoTraderReqDlg::InitializeContinuationCombo()
{
   int row = m_cmbContinuation.AddString(_T("NoContinuation"));
   m_cmbContinuation.SetItemData(row, tsctNoContinuation);

   row = m_cmbContinuation.AddString(_T("Standard"));
   m_cmbContinuation.SetItemData(row, tsctStandard);

   row = m_cmbContinuation.AddString(_T("StandardByMonth"));
   m_cmbContinuation.SetItemData(row, tsctStandardByMonth);

   row = m_cmbContinuation.AddString(_T("Active"));
   m_cmbContinuation.SetItemData(row, tsctActive);

   row = m_cmbContinuation.AddString(_T("ActiveByMonth"));
   m_cmbContinuation.SetItemData(row, tsctActiveByMonth);

   row = m_cmbContinuation.AddString(_T("Adjusted"));
   m_cmbContinuation.SetItemData(row, tsctAdjusted);

   row = m_cmbContinuation.AddString(_T("AdjustedByMonth"));
   m_cmbContinuation.SetItemData(row, tsctAdjustedByMonth);
}

void CAutoTraderReqDlg::OnBnClickedBtnChangeValue()
{
   try
   {
      // Get parameter name by row
      const int row = m_lvParameters.GetSelectionMark();
      CString name = m_lvParameters.GetItemText(row, pcName);

      // Set parameter into the request
      m_spRequest->PutParameter(ATL::CComVariant(name), m_edParameterValue.GetVariantValue());

      // Change parameter value in the list
      m_lvParameters.SetItemText(row, pcValue, 
            Common::VariantToString(m_edParameterValue.GetVariantValue(), CHelper::PARAM_FRAC_PART_LENGTH));
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderReqDlg::OnCbnSelChangeCmbHistorical()
{
   try
   {
      // Get historical period
      eHistoricalPeriod historicalPeriod = 
         static_cast<eHistoricalPeriod>(m_cmbHistorical.GetItemData(m_cmbHistorical.GetCurSel()));

      // Enable/disable intraday period controls 
      m_edIntraday.EnableWindow(hpUndefined == historicalPeriod);
      m_spnIntraday.EnableWindow(hpUndefined == historicalPeriod);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderReqDlg::OnDeltaposSpnIntraday(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edIntraday.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CAutoTraderReqDlg::OnDeltaposSpnDaysBeforeExp(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edDaysBeforeExp.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CAutoTraderReqDlg::OnDeltaposSpnSessionsFilter(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
      m_edSessionsFilter.DecreaseValue(pNMUpDown->iDelta);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CAutoTraderReqDlg::OnBnClickedChkCustomSession()
{
   try
   {
      UpdateData();

      m_spnSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);
      m_edSessionsFilter.ShowWindow(m_CustomSession ? SW_HIDE : SW_SHOW);

      m_edCustomSessionsFilter.ShowWindow(m_CustomSession ? SW_SHOW : SW_HIDE);
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderReqDlg::OnCbnSelChangeCmbContinuation()
{
   try
   {
      // Get continuation type
      eTimeSeriesContinuationType continuationType = 
         static_cast<eTimeSeriesContinuationType>(m_cmbContinuation.GetItemData(m_cmbContinuation.GetCurSel()));

      switch (continuationType)
      {
      case tsctNoContinuation:
      case tsctStandard:
      case tsctStandardByMonth:
         m_chkEqualizeCloses.EnableWindow(FALSE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      case tsctActive:
      case tsctActiveByMonth:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(FALSE);
         m_spnDaysBeforeExp.EnableWindow(FALSE);
         break;
      default:
         m_chkEqualizeCloses.EnableWindow(TRUE);
         m_edDaysBeforeExp.EnableWindow(TRUE);
         m_spnDaysBeforeExp.EnableWindow(TRUE);
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}

void CAutoTraderReqDlg::SetSessionFlags(
                              eSessionFlag sessionFlag)
{
   UpdateData();
   m_DailyFromIntraday = (sfDailyFromIntraday == (sessionFlag & sfDailyFromIntraday));
   UpdateData(FALSE);
}

eSessionFlag CAutoTraderReqDlg::GetSessionFlags()
{
   UpdateData();

   int sessionFlag = sfUndefined;

   if (m_DailyFromIntraday)
   {
      sessionFlag = (sessionFlag | sfDailyFromIntraday);
   }

   return (eSessionFlag)sessionFlag;
}

void CAutoTraderReqDlg::OnLvnItemchangedLsvParameters(
                              NMHDR *pNMHDR,
                              LRESULT *pResult)
{
   try
   {
      // Disable parameter change controls and set value to 0
      m_btnChangeValue.EnableWindow(false);
      m_edParameterValue.EnableWindow(false);
      m_edParameterValue.SetVariantValue(0);

      LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);

      if (LVIS_SELECTED != (pNMLV->uNewState & LVIS_SELECTED))
      {
         // If there is no new item selected then do nothing
         return;
      }

      // Get parameter name
      CString name = m_lvParameters.GetItemText(pNMLV->iItem, pcName);

      // Get parameter current value from request
      ATL::CComVariant value = m_spRequest->GetParameter(ATL::CComVariant(name));

      // Set parameter current value in the edit control
      m_edParameterValue.SetVariantValue(value);

      // Enable parameter change controls
      m_btnChangeValue.EnableWindow(true);
      m_edParameterValue.EnableWindow(true);

      *pResult = 0;
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }
}
