// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <cassert>
#include "Common.h"
#include "Helper.h"
#include "AutoTrader.h"
#include "AutoTraderDlg.h"

STDMETHODIMP CAutoTraderDlg::OnDataError(
                              LPDISPATCH obj,
                              BSTR errorDescription)
{
   UNREFERENCED_PARAMETER(obj);

   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnDataConnectionStatusChanged(
                              eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      if (csConnectionUp == newStatus)
      {
         m_spCQGCEL->RequestTradingSystemDefinitions();
      }

      EnableControls();
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnGWConnectionStatusChanged(
                              eConnectionStatus newStatus)
{
   try
   {
      m_lblGwConnection.SetWindowText(_T("GW ") + CHelper::ConnectionStatusToString(newStatus));

      if (csConnectionUp == newStatus)
      {
         m_spCQGCEL->PutAccountSubscriptionLevel(aslAccountUpdatesAndOrders);
      }

      EnableControls();
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnAccountChanged(
                              eAccountChangeType change_type,
                              ICQGAccount* cqg_account,
                              ICQGPosition* cqg_position)
{
   UNREFERENCED_PARAMETER(cqg_account);
   UNREFERENCED_PARAMETER(cqg_position);

   try
   {
      if (actAccountsReloaded != change_type)
      {
         return S_OK;
      }

      if (m_spCQGCEL->GetAccounts()->GetCount() > 0)
      {
         m_AccountsResolved = true;
      }
      else
      {
         m_AccountsResolved = false;
      }

      m_lblAccountsStatus.SetWindowText(m_AccountsResolved ? 
                                          _T("Accounts Resolved") : _T("Accounts Not Resolved"));

      EnableControls();
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnInstrumentResolved(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      if (m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error)))
      {
         OnResolutionFailed(symbol);
         return S_OK;
      }

      for(Traders::iterator it = m_TradersMap.begin(); it != m_TradersMap.end(); ++it)
      {
         if (((it->second).GetSymbol() == symbol) && (it->second).GetInstrumentName().IsEmpty())
         {
            (it->second).SetInstrument(cqg_instrument);

            const int row = FindAutoTraderRow(it->first);
            if (-1 < row)
            {
               m_lvAutoTraders.SetItemText(row, atcSymbolStatus, _T("Resolved"));
               m_lvAutoTraders.SetItemText(row, atcFullName, ATL::CW2T(cqg_instrument->GetFullName()));
            }
         }
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

void CAutoTraderDlg::OnResolutionFailed(BSTR symbol)
{
   for (Traders::iterator it = m_TradersMap.begin(); it != m_TradersMap.end(); ++it)
   {
      if (((it->second).GetSymbol() == symbol) && (it->second).GetInstrumentName().IsEmpty())
      {
         (it->second).SetInstrumentName(CHelper::N_A);

         const int row = FindAutoTraderRow(it->first);
         if (-1 < row)
         {
            m_lvAutoTraders.SetItemText(row, atcSymbolStatus, _T("Incorrect Symbol"));
            m_lvAutoTraders.SetItemText(row, atcFullName, CHelper::N_A);
         }
      }
   }
}

STDMETHODIMP CAutoTraderDlg::OnTradingSystemDefinitionsResolved(
                              ICQGTradingSystemDefinitions* cqg_tradingSystem_definitions, 
                              ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error));

      if (isValid)
      {
         _bstr_t errDescription = cqg_error->GetDescription();
         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      // Fill up new data
      m_cmbDefinitions.ResetContent();

      long count = cqg_tradingSystem_definitions->GetCount();

      for (long i = 0; i < count; ++i)
      {
         ICQGTradingSystemDefinitionPtr spTradingSystemDefinition = cqg_tradingSystem_definitions->GetItem(i);
         _bstr_t name = spTradingSystemDefinition->GetName();

         const int row = m_cmbDefinitions.AddString(ATL::CW2T(name));
         m_cmbDefinitions.SetItemData(row, i);
      }

      if (0 < count)
      {
         // Select first definition
         m_cmbDefinitions.SetCurSel(0);
         m_DefinitionsResolved = true;
      }

      m_lblDefinitionsStatus.SetWindowText(m_DefinitionsResolved ? 
                                             _T("Definitions Resolved") : _T("Definitions Not Resolved"));

      EnableControls();
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnTradingSystemResolved(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGError* cqg_error)
{
   try
   {
      if (m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error)))
      {
         AfxMessageBox(ATL::CW2T(cqg_error->Description), MB_ICONSTOP);
         return S_OK;
      }

      if (!m_spCQGCEL->IsValid(ATL::CComVariant(cqg_trading_system)))
      {
         AfxMessageBox(_T("Trade system is not resolved."), MB_ICONSTOP);
         return S_OK;
      }

      Trader& trader = GetAutoTraderByID(cqg_trading_system->GetId());
      if (cqg_trading_system->TradesRows &&
          cqg_trading_system->TradesRows->Count > 0)
      {
         trader.SetTradesRowData(cqg_trading_system->TradesRows->GetItem(cqg_trading_system->TradesRows->Count - 1));
      }

      if (trader.TradingSystemChanged())
      {
         ShowOrder(trader.GetName(), trader.GetLastOrder());
      }

      const int row = FindAutoTraderRow(cqg_trading_system->GetId());
      if (-1 < row)
      {
         m_lvAutoTraders.SetItemText(row, atcTradingSystemStatus, 
                                          CHelper::RequestStatusToString(cqg_trading_system->GetStatus()));
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnTradingSystemAddNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemAddInfo* cqg_trading_system_add_info)
{
   UNREFERENCED_PARAMETER(cqg_trading_system_add_info);

   try
   {
      Trader& trader = GetAutoTraderByID(cqg_trading_system->GetId());
      Common::TradesRowData tradesRowData;
      ICQGTradingSystemTradesRowPtr spTradesRow = 
         cqg_trading_system->TradesRows->GetItem(cqg_trading_system->TradesRows->Count - 1);
      tradesRowData.m_timestamp = spTradesRow->Timestamp;
      tradesRowData.m_timestampOffset = spTradesRow->TimestampOffset;
      if (Common::Compare(trader.GetTradesRowData(), tradesRowData) <= 0 && trader.TradingSystemChanged())
      {
         ShowOrder(trader.GetName(), trader.GetLastOrder());
      }

      const int row = FindAutoTraderRow(cqg_trading_system->GetId());
      if (-1 < row)
      {
         m_lvAutoTraders.SetItemText(row, atcOpenPositions, trader.GetOpenPositionStr());
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnTradingSystemUpdateNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemUpdateInfo* cqg_trading_system_update_info)
{
   UNREFERENCED_PARAMETER(cqg_trading_system_update_info);

   try
   {
      Trader& trader = GetAutoTraderByID(cqg_trading_system->GetId());
      Common::TradesRowData tradesRowData;
      ICQGTradingSystemTradesRowPtr spTradesRow = 
         cqg_trading_system->TradesRows->GetItem(cqg_trading_system->TradesRows->Count - 1);
      tradesRowData.m_timestamp = spTradesRow->Timestamp;
      tradesRowData.m_timestampOffset = spTradesRow->TimestampOffset;
      if (Common::Compare(trader.GetTradesRowData(), tradesRowData) <= 0 && trader.TradingSystemChanged())
      {
         ShowOrder(trader.GetName(), trader.GetLastOrder());
      }

      const int row = FindAutoTraderRow(cqg_trading_system->GetId());
      if (-1 < row)
      {
         m_lvAutoTraders.SetItemText(row, atcOpenPositions, trader.GetOpenPositionStr());
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CAutoTraderDlg::OnOrderChanged(
                              eChangeType change, 
                              ICQGOrder* order, 
                              ICQGOrderProperties* oldProperties, 
                              ICQGFill* fill, 
                              ICQGError* cqgerr)
{
   UNREFERENCED_PARAMETER(change);
   UNREFERENCED_PARAMETER(oldProperties);
   UNREFERENCED_PARAMETER(fill);
   UNREFERENCED_PARAMETER(cqgerr);

   try
   {
      if (order)
      {
         const int row = FindOrderRow(order->GetGUID());
         if (-1 < row)
         {
            UpdateOrder(row, order);
         }
      }
   }
   catch (_com_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.Description()), MB_ICONSTOP);
   }

   return S_OK;
}