// Common.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"

CString Common::DateTimeToString(
                        DATE dateTime, 
                        bool withTime/* = true*/,
                        bool withDate/* = true*/)
{
   COleDateTime oleDateTime(dateTime);
   CString dateTimeString;

   if (withDate)
   {
      dateTimeString = oleDateTime.Format(_T("%d/%m/%Y"));
   }

   if (withTime)
   {
      CString timeString = oleDateTime.Format(_T("%H:%M"));
      dateTimeString += withDate ? " " : "";
      dateTimeString += timeString;
   }

   return dateTimeString;
}

CString Common::VariantToString(
                        const ATL::CComVariant& value)
{
   CString strValue;
   switch (value.vt)
   {
   case VT_I4:
      strValue.Format(_T("%d"), value.intVal);
      break;
   case VT_UI4:
      strValue.Format(_T("%d"), value.uintVal);
      break;
   case VT_R8:
      strValue.Format(_T("%.2f"), value.dblVal);
      break;
   case VT_DATE:
      strValue = DateTimeToString(value.date);
      break;
   case VT_BSTR:
      strValue = value.bstrVal;
      break;
   case VT_I2:
   case VT_BOOL:
      strValue = value.boolVal ? "True" : "False";
      break;
   default:
      throw std::invalid_argument("Unknown variant type");
   }

   return strValue;
}

CString Common::VariantToString(
                        const ATL::CComVariant& value,
                        int fracPart)
{
   if (VT_R8 == value.vt)
   {
      CString formatStr;
      formatStr.Format(_T("%%.%df"), fracPart);
      CString strValue;
      strValue.Format(formatStr, value.dblVal);

      return strValue;
   }
   else
   {
      return VariantToString(value);
   }
}

CString Common::VariantToString(
                        const ATL::CComVariant& value,
                        bool withTime)
{
   if (VT_DATE == value.vt)
   {
      return DateTimeToString(value.date, withTime);
   }
   else
   {
      return VariantToString(value);
   }
}

CString Common::VariantToString(
                        const ATL::CComVariant& value,
                        bool withTime,
                        bool withDate)
{
   if (VT_DATE == value.vt)
   {
      return DateTimeToString(value.date, withTime, withDate);
   }
   else
   {
      return VariantToString(value);
   }
}

int Common::Compare(const TradesRowData& first, const TradesRowData& second)
{
   if (first.m_timestamp < second.m_timestamp)
   {
      return -1;
   }
   else if (first.m_timestamp > second.m_timestamp)
   {
      return 1;
   }
   else
   {
      if (first.m_timestampOffset < second.m_timestampOffset)
      {
         return -1;
      }
      else if (first.m_timestampOffset > second.m_timestampOffset)
      {
         return 1;
      }
   }

   return 0;
}