/*
* =====================================================================
*  Filename: Helper.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

class CHelper
{

private:

   /// \brief CHelper constructor.
   CHelper();

public:

   /// \brief Converts enum of connection status to appropriate string.
   /// \param connectionStatus - enum which will be converted
   /// \return String which describes connection status.
   static CString ConnectionStatusToString(
						eConnectionStatus connectionStatus);

   /// \brief Converts enum of parameter type to appropriate string.
   /// \param parameterType - enum which will be converted
   /// \return String which describes parameter type.
   static CString ParameterTypeToString(
                  eUserFormulaParameterType parameterType);

   /// \brief Converts enum of request status to appropriate string.
   /// \param requestStatus - enum which will be converted
   /// \return String which describes request status.
   static CString RequestStatusToString(
						eRequestStatus requestStatus);

   /// \brief Converts enum of historical period to appropriate string.
   /// \param historicalPeriod - enum which will be converted
   /// \return String which describes historical period.
   static CString HistoricalPeriodToString(
						eHistoricalPeriod historicalPeriod);

   /// \brief Converts enum of continuation type to appropriate string.
   /// \param continuationType - enum which will be converted
   /// \return String which describes continuation type.
   static CString ContinuationTypeToString(
						eTimeSeriesContinuationType continuationType);

   /// \brief Converts enum of continuation type to appropriate short string.
   /// \param continuationType - enum which will be converted
   /// \return String which describes continuation type.
   static CString ContinuationTypeToShortString(
                  eTimeSeriesContinuationType continuationType);

   /// \brief Converts bitmap of session flag to appropriate string.
   /// \param sessionFlag - bitmap which will be converted
   /// \return String which describes session flag.
   static CString SessionFlagToString(
						eSessionFlag sessionFlag);

   /// \brief Converts enum of order type to appropriate string.
   /// \param orderType - enum which will be converted
   /// \return String which describes order type.
   static CString OrderTypeToString(
                  eOrderType orderType);
   
   /// \brief Returns string representation of the specified order side.
   /// \param side - order side
   /// \return String which describes order side.
   static CString OrderSideToString(
                  eOrderSide side);

   /// \brief Returns string representation of the specified order status.
   /// \param status - order status
   /// \return String which describes order status.
   static CString OrderStatusToString(
                  eOrderStatus status);

   /// \brief Returns string representation of the specified order state.
   /// \param state - order state
   /// \return String which describes order state.
   static CString OrderStateToString(
                  eOrderLocalState state);

public:

   /// \brief Selects an item of the specified combo box, which item data matches with given.
   /// \param comboBoxCtrl - combo box, which item will be selected
   /// \param itemData - item data, which must match with selected item data
   static void SetComboBoxItem(
						CComboBox& comboBoxCtrl,
						DWORD_PTR itemData);
public:

   /// \brief Custom date format.
   static const CString DATE_FORMAT;

   /// \brief Not available string.
   static const CString N_A;

   /// \brief Not resolved string.
   static const CString NOT_RESOLVED;

   /// \brief UEName string for auto traded order.
   static const CString UE_NAME;

   /// \brief Parameter fractional part length.
   static const int PARAM_FRAC_PART_LENGTH;
};
