/*
* =====================================================================
*  Filename: Trader.h
*    Author: Sevada Gyurjyan
*   Company: CQG, Inc.
* Copyright: (c)2008 by CQG,Inc. All rights reserved. 
* =====================================================================
*/

#pragma once

#include "Common.h"

class Trader
{

public:
   
   Trader(
      const ICQGCELPtr& spCQGCEL,
      const CString& autoTraderName,
      long accountID,
      const ICQGTradingSystemPtr& spTradingSystem);
   
   virtual ~Trader(void);

public:

   long GetAccountID() const {return m_AccountID;}

   void SetAccountID(long val) {m_AccountID = val;}

   const CString& GetInstrumentName() const {return m_InstrumentName;}

   void SetInstrumentName(const CString& val) {m_InstrumentName = val;}

   const CString& GetSymbol() {return m_Symbol.MakeUpper();}

   void SetSymbol(const CString& val) {m_Symbol = val;}

   const CString& GetName() const {return m_Name;}

   void SetName(const CString& val) {m_Name = val;}

   const Common::TradesRowData& GetTradesRowData() const { return m_tradesRowData; }

   void SetTradesRowData(const ICQGTradingSystemTradesRowPtr& spTradesRow) 
   { 
      m_tradesRowData.m_timestamp = spTradesRow->Timestamp;
      m_tradesRowData.m_timestampOffset = spTradesRow->TimestampOffset;
   }

public:

   void Activate(bool status);

   bool TradingSystemChanged();

   void PlaceOrder(long quantity);

   void SetInstrument(const ICQGInstrumentPtr& val);

   CString GetActivationTime();

   CString GetAccountName();

   CString GetOpenPositionStr();

   CString GetPreferences();

   CString GetParameters();

   ICQGAccountPtr GetAccount();

   ICQGOrderPtr GetLastOrder();

   bool GetReadyStatus();

private:

   long m_OpenPositions;
   
   long m_AccountID;

   COleDateTime m_ActivationTime;
 
   CString m_Symbol;
   
   CString m_InstrumentName;
   
   CString m_Name;
   
   ICQGCELPtr m_spCQGCEL;

   ICQGTradingSystemPtr m_spTradingSystem;

   ICQGInstrumentPtr m_Instrument;

   ICQGOrderPtr m_LastOrder;

   bool m_IsActivated;

   Common::TradesRowData m_tradesRowData;

};